/** This module handles stored options, currently all except input maps.
*
* The purpose of having all options centrally controlled is to allow generic handling by the GUI
* and ease saving and loading of values.
*/
module mde.options;

import mde.exception;

import mde.mergetag.read;
import mde.mergetag.write;
import mde.mergetag.dataset;
import mde.mergetag.exception;

import tango.scrapple.text.convert.parseTo : parseTo;
import tango.scrapple.text.convert.parseFrom : parseFrom;

import tango.io.FilePath : FilePath;
import tango.util.log.Log : Log, Logger;

private Logger logger;
static this() {
    logger = Log.getLogger ("mde.options");
}

Options options;

// Later: add automatic saving/loading for variables, variable lists with GUI name and description.

/** Class instance stores all options.
*
* All options and handling should be non-static to allow possibility of profiles later.
*/
class Options : DataSection
{
    /* The options.
    *
    * For now, static variables. Later, lists (and static variables?).
    */
    bool useThreads;
    char[] greeting;    // just a testing message
    
    
    /* The code to load/save the values.
    *
    * Uses mergetag.
    */
    void addTag (char[] tp, ID id, char[] dt) {
        if (tp == "bool") {
            if (id == cast(ID)"useThreads") useThreads = parseTo!(bool) (dt);
        } else if (tp == "char[]") {
            if (id == cast(ID)"greeting") greeting = parseTo!(char[]) (dt);
        } else
            throw new MTUnknownTypeException ("Only bool currently supported.");
    }
    void writeAll (ItemDelg dlg) {
        dlg ("bool", cast(ID)"useThreads", parseFrom!(bool) (useThreads));
        dlg ("char[]", cast(ID)"greeting", parseFrom!(char[]) (greeting));
    }
    
    
    /* Load/save options from file.
    *
    * If the file doesn't exist, no reading is attempted (options are left at default values).
    */
    private static const fileName = "conf/options.mtt";
    private static const secName = cast(ID)"Default";
    static void load () {
        scope(failure) options = new Options();     // FIXME necessary while options is accessed statically and access is unprotected!
        
        FilePath filePath = new FilePath (fileName);
        // If it's not a file, stop. It should still be created on exit (assuming it's not a folder).
        if (!filePath.exists || filePath.fileSize == 0u) {
            options = new Options();
            return;
        }
        
        Reader reader;
        try {
            reader = new Reader(filePath);
            reader.dataSecCreator = function DataSection(ID) {
                return new Options;
            };
            reader.read;
        } catch (MTException e) {
            logger.error ("Mergetag exception occurred:");
            logger.error (e.msg);
            throw new optionsLoadException ("Loading aborted: mergetag exception");
        }
        
        DataSection* secP = secName in reader.dataset.sec;
        options = cast(Options) *secP;
        if (options is null) {
            throw new optionsLoadException ("Loading failed: expected section not found");
        }
        // else loading was succesful.
    }
    static void save () {
        DataSet ds = new DataSet();
        ds.sec[secName] = options;
        
        IWriter writer;
        try {
            writer = makeWriter (fileName, ds);
            writer.write();
        } catch (MTException e) {
            logger.error ("Mergetag exception occurred; saving aborted:");
            logger.error (e.msg);
            //FIXME: currently nothing done besides logging the error
        }
    }
}
